--------------------------------------------------------------------------------
--            NOKIA GRAPHICS LCD DISPLAY DRIVER TO PRINTER PORT
--Author  : Sotiris Bellos     <belltron@mail.com>
--Location: ARTA - GREECE
--Work    : Electronics - Copmuters - mobile phones  Repairs
--Project : Nokia LCD driver(part of my W.I.P car mp3player running Euphoria)
--Version : 0.1 (first release  --quick and dirty--)
--Date    : 14-Aug-2003
--Credits : Jacques Deschenes for ports.e - delay.e
--          Rapideuphoria for Euphoria programming language
--Lang ver: Euphoria v2.1
--Next version will support (95% ready)
	--Jiri Babor's font files
	--BMP monochrome files
	--Logomanager .NOL files
	--Logomanager .NGG files
	--Logomanager .NSL files
	--Logomamager .NLM animations
	--Fonts extracted from mobile phone flash files
						--sorry for my poor english--
--------------------------------------------------------------------------------
include ports.e
include delay.e
include misc.e
constant POW=1,
	 CLK=2,
	 DIN=4,
	 DC=8,
	 CE=16,
	 RES=32
constant adr=#378
sequence lcd_ram,ram_loc
integer update
lcd_ram=repeat({0,0,0,0,0,0,0,0},6*84)
ram_loc={}
--------------------------------------------------------------------------------

--function set 32 = chip is active - horizontial addressing - basic instr set
--             33 =                                           exten instr set
--             34 =                  vertical addressing
--             36 = power down mode
------------basic instruction set
--display control 08 = display blank
--                09 = all display segments on
--                12 = normal mode
--                13 = inverse video mode
--
--set y address of ram  64+ 0<=y<=5
--set x address of ram  128+ 0<=x<=83
-----------extended instruction set
--temperature control 4 - 7 set temper coefficient
--bias system 16 - 23 set bias system
--set Vop 128-255 write Vop to register

-- For more info about controller chip look at PCD8544.PDF
--------------------------------------------------------------------------------
procedure set(integer pin) -- sets output bit HI
   Output (or_bits(Input(adr),pin),adr)
end procedure
--------------------------------------------------------------------------------
 procedure reset(integer pin) -- sets output bit LOW
   Output (and_bits(Input(adr),255-pin),adr)
end procedure
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
procedure lcd_serial(object d) --sent to LCD a byte in serial form
    sequence dd
  if sequence(d) then
     dd=reverse(d)
     else
    dd=int_to_bits(d,8)
   end if
    for i=8 to 1 by -1 do
   reset(CLK)
    if dd[i]=0 then reset(DIN) else  set(DIN) end if
    set(CLK)
    end for
end procedure
--------------------------------------------------------------------------------
global procedure lcd_cls()  -- clear the display
reset(DIN)
set(DC)
reset(CE)
for i=6 to 1 by -1 do
    for p=84 to 1 by -1  do
    for t=8 to 1 by -1 do
     reset(CLK)
     set(CLK)
    end for
    end for
end for
lcd_ram=repeat({0,0,0,0,0,0,0,0},6*84)
ram_loc={}
set(CE)
end procedure
--------------------------------------------------------------------------------
global procedure lcd_cmd(atom d) -- command byte send to LCD
reset(DC)
reset(CE)
lcd_serial(d)
set(CE)
end procedure
--------------------------------------------------------------------------------
global procedure lcd_dat(atom d) --data byte send to LCD
set(DC)
reset(CE)
lcd_serial(d)
set(CE)
end procedure
--------------------------------------------------------------------------------
global procedure lcd_xy(integer x,integer y) --goto location x,y
lcd_cmd(y+#40)                               --x range 0-83
lcd_cmd(x+#80)                               --y range 0-5
end procedure
--------------------------------------------------------------------------------
global procedure lcd_pixel(integer x,integer y,integer mode)
    integer q,l,b,u          -- set or clear a pixel in the LCD
    u=0                      -- x range 1 - 84
     q=floor((y-1)/8)        -- y range 1 - 48
    l=(q*84)+x
    b=y-(q*8)
if mode=0  and lcd_ram[l][9-b]=1 then
  lcd_ram[l][9-b]=0
  u=1
  end if
if mode=1 and lcd_ram[l][9-b]=0 then
   lcd_ram[l][9-b]=1
   u=1
  end if
if u=1 then
  if not find(l,ram_loc) then
  ram_loc=append(ram_loc,l)
end if
end if
end procedure
--------------------------------------------------------------------------------
global procedure lcd_xline(integer x1,integer x2,integer y,integer mode)
for p=x1 to x2 do          -- draw a horizontial line
lcd_pixel(p,y,mode)
end for
call_proc(update,{})
end procedure
--------------------------------------------------------------------------------
global procedure lcd_yline(integer y1,integer y2,integer x,integer mode)
for p=y1 to y2 do        --draw a vertical line
lcd_pixel(x,p,mode)
end for
call_proc(update,{})
end procedure
--------------------------------------------------------------------------------
global procedure lcd_box(integer x1,integer y1,integer x2,integer y2,integer mode)
if mode=0 then              --draw a box
for i=y1 to y2 do                  -- mode 0 = inverted box
    for t=x1 to x2 do              -- mode 1 = outline
    lcd_pixel(t,i,mode)            -- mode 2 = filled
    end for
end for
end if
if mode=1 then
    for i=x1 to x2 do
    lcd_pixel(i,y1,mode)
    end for
    for i=x1 to x2 do
    lcd_pixel(i,y2,mode)
    end for
    for i=y1 to y2 do
    lcd_pixel(x1,i,mode)
    end for
    for i=y1 to y2 do
    lcd_pixel(x2,i,mode)
    end for
end if
if mode=2 then
for i=y1 to y2 do
    for t=x1 to x2 do
    lcd_pixel(t,i,1)
    end for
end for
end if
call_proc(update,{})
end procedure
--------------------------------------------------------------------------------
global procedure lcd_update()    -- update the lcd screen after modifycation
integer x1,y1,l
if length(ram_loc)>0 then
   for i=1 to length(ram_loc) do
   l=ram_loc[i]
   y1=floor(l/84)
   if l-(y1*84)=0 then y1=y1-1 end if
   x1=(l-(y1*84))-1
   if x1<0 then y1=y1-1 x1=x1+84 end if
   lcd_xy(x1,y1)
set(DC)
reset(CE)
lcd_serial(lcd_ram[l])
set(CE)
   end for
ram_loc={}
end if
end procedure
update=routine_id("lcd_update")
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
global procedure lcd_init()   -- initialize the display
set(POW)
delay(0.006)
reset(RES)
delay(0.25)
set(RES)
delay(0.25)
lcd_cmd(#21)
lcd_cmd(#98)
lcd_cmd(#06)
lcd_cmd(#13)
lcd_cmd(#20)
lcd_cmd(#0C)
lcd_cls()
lcd_xy(0,0)
end procedure
--------------------------------------------------------------------------------
Output(0,adr)
Output(1+4+8+16+32,adr)

